#!/usr/bin/perl
# kml to .map converter v1.0 - N.Kozawa 2007/06/22
# This tool does not support linestring across date line

# preset values
$mapType = "GoogleMap";
$zoomLevel = "00-14";
$hSpace = $vSpace = 0.1;  # 100m
$nmile = 1.852; # (Km)
$ver = "v1.0";

while (0 < $#ARGV) {
	$opt = shift;
	if ($opt =~ /^(--m|mt|-maptype)$/i) {
		$mapType = shift;
	} elsif ($opt =~ /^(-z|-zoom|-zoomlevel)$/i) {
		$zoomLevel = shift;
	} elsif ($opt =~ /^(-h|-hspace|-margin)$/i) {
		$hSpace = shift;
	} elsif ($opt =~ /^(-v|-vspace|-vmargin)$/i) {
		$vSpace = shift;
	} elsif ($opt =~ /^-/) {
		print "Usage: kml2map [OPTION] filename.kml\n";
		print " -m, -mt, -maptype         Map Type\n";
		print " -z, -zoom, -zoomlevel     Zoom Level\n";
		print " -h, -hspace, -hmargin     Horizontal Margin\n";
		print " -v, -vspace, -vmargin     Vertical Margin\n";
		exit;
	}
}

# init vars for while loop
$coord = $linestring = 0;
$maxLong = -180;
$minLong = 180;
$maxLat = -90;
$minLat = 90;
while (<>) {
	if (/<LineString>/i) { $linestring = 1; }
	if (/<\/LineString>/i) { $linestring = 0; }
	if (1 == $linestring && /<coordinates>/i) {
		$coord = 1;
		s/<coordinates>//i;
	}
	if (1 == $coord) {
		s/[\n\r]//g;
		($long, $lat) = split(",");
		if ($long =~ /\d/ && $lat =~ /\d/) {
			if ($minLong > $long) { $minLong = $long; }
			if ($maxLong < $long) { $maxLong = $long; }
			if ($minLat > $lat) { $minLat = $lat; }
			if ($maxLat < $lat) { $maxLat = $lat; }
		}
	}
	if (/<\/coordinates>/i) { $coord = 0; }
}
$offsetLat = $vSpace / ($nmile * 60);
$tOffsetLong = (90 > $maxLat) ? $hSpace / ($nmile * 60 * (90 - $maxLat) / 90) : 0;
$bOffsetLong = (90 > $minLat) ? $hSpace / ($nmile * 60 * (90 - $minLat) / 90) : 0;
$tLat = $maxLat + $offsetLat;
if (90 < $tLat) { $tLat = 90; }
$lLong = $minLong - $tOffsetLong;
if (-180 > $lLong) { $lLong = -180; }
$bLat = $minLat - $offsetLat;
if (-90 > $bLat) { $bLat = -90; }
$rLong = $maxLong + $bOffsetLong;
if (180 < $rLong) { $rLong = 180; }
print "# Map definition file generated by kml2map $ver.\n";
print "$mapType\n";
printf("$zoomLevel : %.6f, %.6f : %.6f, %.6f\n", $tLat, $lLong, $bLat, $rLong);
